package com.diagbot.facade;

import com.diagbot.dto.ImgVerCreatDTO;
import com.diagbot.entity.ImgVerInfo;
import com.diagbot.entity.SwsVerInfo;
import com.diagbot.entity.User;
import com.diagbot.enums.VisibleIdTypeEnum;
import com.diagbot.exception.CommonException;
import com.diagbot.exception.ErrorCode;
import com.diagbot.idc.VisibleIdCreater;
import com.diagbot.service.ImgVerService;
import com.diagbot.service.SmsVerService;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.RegexValidateUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ImgVerVerVO;
import com.diagbot.vo.SmsVerCreatVO;
import com.diagbot.vo.SmsVerVerVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @Description: 用户验证业务层
 * @author: gaodm
 * @time: 2018/9/4 19:31
 */
@Component
public class VerFacade {
    @Autowired
    private ImgVerService imgVerService;
    @Autowired
    private SmsVerService smsVerService;
    @Autowired
    private UserFacade userFacade;

    /**
     * 获取用户图片验证信息
     * @return 图片验证码信息
     */
    public ImgVerCreatDTO getImgVerification(){
        ImgVerCreatDTO imgVerCreatDTO = new ImgVerCreatDTO();
        VisibleIdCreater visibleIdCreater = new VisibleIdCreater(1,1);
        String imgId = visibleIdCreater.getNextId(VisibleIdTypeEnum.IS_IMG_VER.getKey()).toString();
        ImgVerInfo imgVerInfo = new ImgVerInfo();
        imgVerInfo.setImgId(imgId);
        byte[] res = imgVerService.createImgVerification(imgVerInfo);
        imgVerCreatDTO.setImgId(imgId);
        imgVerCreatDTO.setImgByte(res);
        return imgVerCreatDTO;
    }

    /**
     * 验证用户图片验证码是否有效
     * @param imgVerVerVO 图片验证码验证输入信息
     * @return 验证是否成功
     */
    public Boolean verifyImgVerification(ImgVerVerVO imgVerVerVO){
        ImgVerInfo imgVerInfo = new ImgVerInfo();
        BeanUtil.copyProperties(imgVerVerVO, imgVerInfo);
        return imgVerService.verifyImgVerification(imgVerInfo);
    }

    /**
     * 获取短信验证码
     * @param smsVerCreatVO 获取短信验证码输入参数
     * @return 获取短信验证码是否成功
     */
    public Boolean getSmsVerification(SmsVerCreatVO smsVerCreatVO){
        //参数验证
        if (null == smsVerCreatVO){
            throw new CommonException(ErrorCode.PARAM_IS_NULL,
                    "短信验证码创建信息不能为空!");
        }
        if (StringUtil.isBlank(smsVerCreatVO.getImgId())){
            throw new CommonException(ErrorCode.PARAM_IS_NULL,
                    "图片验证码唯一标志不能为空!");
        }
        if (StringUtil.isBlank(smsVerCreatVO.getCode())){
            throw new CommonException(ErrorCode.PARAM_IS_NULL,
                    "图片验证码不能为空!");
        }
        if (StringUtil.isBlank(smsVerCreatVO.getMobile())){
            throw new CommonException(ErrorCode.PARAM_IS_NULL,
                    "用户电话不能为空!");
        }
        if (!RegexValidateUtil.checkMobileNumber(smsVerCreatVO.getMobile())){
            throw new CommonException(ErrorCode.PARAM_IS_ERROR,
                    "用户电话格式错误!");
        }
        //验证图片验证码
        ImgVerInfo imgVerInfo = new ImgVerInfo();
        BeanUtil.copyProperties(smsVerCreatVO, imgVerInfo);
        Boolean imgVerRes = imgVerService.verifyImgVerification(imgVerInfo);
        if (imgVerRes){
            //todo 验证该用户是否被注册
            Boolean isReg = false;
            if (isReg){
                throw new CommonException(ErrorCode.PARAM_IS_ERROR, "该号码已经被注册!");
            }

            //验证是否超过两分钟
            SwsVerInfo swsVerInfoCheck = smsVerService.getSmsVerification(smsVerCreatVO.getMobile());
            if (null != swsVerInfoCheck){
                if (DateUtil.after(DateUtil.addMinutes(swsVerInfoCheck.getCreateTime(), 2),
                        DateUtil.now())){
                    throw new CommonException(ErrorCode.SMS_SEND_ERROR,
                            "同一用户两分钟之内不能重复发送验证短信，请两分钟之后再次发送!");
                }
            }
        } else {
            throw new CommonException(ErrorCode.PARAM_IS_ERROR, "图片验证码错误!");
        }

        //发送短信
        String smsTemplateCode = "SMS_143861902";
        String code = smsVerService.smsSend(smsVerCreatVO.getMobile(), smsTemplateCode);

        //创建用户短信验证信息，缓存到redis
        SwsVerInfo swsVerInfo = new SwsVerInfo();
        swsVerInfo.setMobile(smsVerCreatVO.getMobile());
        swsVerInfo.setCode(code);
        return smsVerService.createSmsVerification(swsVerInfo);
    }

    /**
     * 验证短信码并且注册为新的用户
     * @param smsVerVerVO 短信验证码验证输入信息
     * @return 用户信息是否注册到数据库
     */
    public Boolean verifySmsVerification(SmsVerVerVO smsVerVerVO){
        //参数验证
        if (null == smsVerVerVO){
            throw new CommonException(ErrorCode.PARAM_IS_NULL,
                    "短信验证码验证输入信息不能为空!");
        }
        if (StringUtil.isBlank(smsVerVerVO.getImgId())){
            throw new CommonException(ErrorCode.PARAM_IS_NULL,
                    "图片验证码唯一标志不能为空!");
        }
        if (StringUtil.isBlank(smsVerVerVO.getCode())){
            throw new CommonException(ErrorCode.PARAM_IS_NULL,
                    "图片验证码不能为空!");
        }
        if (StringUtil.isBlank(smsVerVerVO.getMobile())){
            throw new CommonException(ErrorCode.PARAM_IS_NULL,
                    "用户电话不能为空!");
        }
        if (StringUtil.isBlank(smsVerVerVO.getSmsCode())){
            throw new CommonException(ErrorCode.PARAM_IS_NULL,
                    "短信验证码不能为空!");
        }
        if (!RegexValidateUtil.checkMobileNumber(smsVerVerVO.getMobile())){
            throw new CommonException(ErrorCode.PARAM_IS_ERROR,
                    "用户电话格式错误!");
        }
        //验证图片验证码
        ImgVerInfo imgVerInfo = new ImgVerInfo();
        BeanUtil.copyProperties(smsVerVerVO, imgVerInfo);
        Boolean imgVerRes = imgVerService.verifyImgVerification(imgVerInfo);
        if (imgVerRes){
            //todo 验证该用户是否被注册
            Boolean isReg = false;
            if (isReg){
                throw new CommonException(ErrorCode.PARAM_IS_ERROR, "该号码已经被注册!");
            }
        } else {
            throw new CommonException(ErrorCode.PARAM_IS_ERROR, "图片验证码错误!");
        }
        //验证用户短信信息
        SwsVerInfo swsVerInfo = new SwsVerInfo();
        swsVerInfo.setMobile(smsVerVerVO.getMobile());
        swsVerInfo.setCode(smsVerVerVO.getSmsCode());
        Boolean smsVer = smsVerService.verifySmsVerification(swsVerInfo);
        if (!smsVer){
            throw new CommonException(ErrorCode.PARAM_IS_ERROR, "短信验证未通过!");
        }
        //todo 将用户信息加入数据库
        Boolean registSuccess = true;
        return registSuccess;
    }


}
